@@ -431,11 +431,7 @@ static int do_write(struct abstract_write *w, UINT32 length, UINT8 *data)
 
 	int res = write(self->fd, data, length);
 
-	if (!res)
-	{
-		return ST_OK | ST_CLOSE;
-	}
-	else if (res < 0)
+	if (res < 0)
 	{
 		switch(errno)
 		{
@@ -445,13 +441,9 @@ static int do_write(struct abstract_write *w, UINT32 length, UINT8 *data)
 		default:
 			werror("io.c: do_write: write() failed (errno %i), %z\n",
 			       errno, strerror(errno));
-			return ST_CLOSE | ST_FAIL;
 		}
 	}
-	else
-	{
-		return ST_OK | ST_GOON;
-	}
+	return res;
 }
 
 static void write_callback(struct nonblocking_fd *fd)
@@ -487,13 +479,18 @@ static void listen_callback(struct nonblocking_fd *fd)
 		werror("io.c: accept() failed, %z", strerror(errno));
 		return;
 	}
-	res = FD_LISTEN_CALLBACK(self->callback, conn,
-				 sockaddr2address_info(addr_len, peer));
-	if (ST_CLOSEDP(res))
-	{
-		closekill_fd(fd, (ST_FAILUREP(res)
-				  ? CLOSE_PROTOCOL_FAILURE
-				  : CLOSE_EOF));
+	if (addr_len) {
+		res = FD_LISTEN_CALLBACK(self->callback, conn,
+					 sockaddr2address_info(addr_len, peer));
+		if (ST_CLOSEDP(res))
+		{
+			closekill_fd(fd, (ST_FAILUREP(res)
+					  ? CLOSE_PROTOCOL_FAILURE
+					  : CLOSE_EOF));
+		}
+	}
+	else {
+		closekill_fd(fd, CLOSE_EOF);
 	}
 }
 
