@@ -91,16 +91,25 @@ static int do_flush(struct abstract_buffer *c, struct abstract_write *w)
 	FOR_QUEUE(&self->queue, struct buffer_node *, item) {
 
 		res = A_WRITE(w, item->packet->length, item->packet->data);
-		if (!ST_CLOSEDP(res)) {
+		if (res >= 0) {
 			if (self->super.writable)
 				(*self->super.writable) = 1;
-			self->queue_size--;
-			ol_queue_remove((struct ol_queue_node *) item);
-			ol_string_free(item->packet);
-			ol_space_free(item);
+			if (item->packet->length == res) {
+				self->queue_size--;
+				ol_queue_remove((struct ol_queue_node *) item);
+				ol_string_free(item->packet);
+				ol_space_free(item);
+			}
+			else {
+				struct ol_string *s;
+				s = item->packet;
+				item->packet = c_format("%s", s->length - res, s->data + res);
+				ol_string_free(s);
+			}
 		}
 		else {
-			return res;
+			werror("pkt_buffer::do_flush(): Error flushing data\n");
+			return ST_FAIL | ST_CLOSE;
 		}
 	}
 	return ST_OK | ST_GOON;
