@@ -197,12 +197,12 @@ static int do_flush(struct abstract_buffer *c, struct abstract_write *w)
 		self->block_size);
                
 	res = A_WRITE(w, size, self->buffer + self->start);
-	if (!ST_FAILUREP(res)) {
-		self->start += size;
+	if (res >= 0) {
+		self->start += res;
 		assert(self->start <= self->end);
-		self->length -= size;
+		self->length -= res;
 	}
-	return 0;
+	return ST_OK | ST_GOON;
 }
 
 void do_close(struct abstract_buffer *c)
